/* **************************************************************************************************
*   MONOSTEREO_DELAY.ASM                                                                            *
*                                                                                                   *
*   Slapback ('Doubling') Effect (also called Mono Automatic Double Tracking)                       *
*   One popular use of the digital delay is to quickly repeat the input signal with a single        *
*   reflection at unity gain. By making the delay an input signal around 15-40 milliseconds,        *
*   the resulting output produces a "slapback/doubling " effect.  The slight differences in the     *
*   delay create the effect of the two parts being played in unison.  The effect is created by      *
*   adding a delayed signal together with the original, where a single input source is mixed with   *
*   the delay, and the result is sent to both output channels.  With short delays, slapback can     * 
*   thicken the sound of an instrument or voice when mixed for a mono result, although              *
*   cancellations can occur from comb filtering side effects when the delay is under 10 ms,         *
*   which will result in a hollow, resonant sound.  Longer slapback delay effects were commonly     *
*   used in the 1950's/1960's music industry.                                                       *
*                                                                                                   *
*                                                                                                   *
*   Mono ADT I/O Equation and Structure:                                                            *
*                           y(n) = x(n) - ax(n - D)                                                 *
*                           yLeft(n) = yRight(n) = y(n)                                             *
*                                                                                                   *
*        x(n) ------------------------------------------|>------>O--------> y(n)                    *
*               |                                       1/2      ^                                  *
*               |                                                |                                  *
*               |                _______________                 |                                  *
*               |               |               |                |                                  *
*               |               |               |                |                                  *
*               |-------------->|     Z^(-D)    |------|>--------|                                  *
*                               |               |      1/2                                          *
*                               |_______________|                                                   *
*                                                                                                   *
*                                                                                                   *
*   Stereo Automatic Double Tracking (Stereo Doubling) - "Concert Announcer Simulation"             *
*   Automatic Double Tracking, which is used by audio engineers in the music industry to create     *
*   a sense of spaciousness in stereophonic systems.   This effect is set up to playback the        *
*   original "dry" signal in one stereo channel and the delayed signal in the other channel.        *
*   This creates the impression of a stereo effect using a single mono source. By making the        *
*   delay an input signal around 15-40 milliseconds, the resulting output produces a                *
*   "stereo doubling" effect.                                                                       *
*                                                                                                   *
*                                                                                                   *
*   Stereo ADT I/O Equation and Structure:                                                          *
*                                        yLeft(n) = x(n)                                            *
*                                        yRight(n) = x(n - D)                                       *
*                                                                                                   *
*       x(n) --------------------------------------------------|>-----------> yLeft(n)              *
*                      |                                        1/2                                 *
*                      |			                                                                *
*                      |             _______________                                                *
*                      |            |               |                                               *
*                      |            |               |                                               *
*                      |----------->|     Z^(-D)    |-----------|>-----------> yRight(n)            *
*                                   |               |           1/2                                 *
*                                   |_______________|                                               *
*                                                                                                   *
*                                                                                                   *
*                                                                                                   *
*   These effects are great for 'spicing' up an instrument or vocal track, such as compensating     *
*   for a singer who is slightly out of key.  To truly simulate a doubling effect, some random      *
*   delay line modulation can be added, since 2 voices are never truely in sync with one another.   *
*                                                                                                   *
*   IRQ1 pushbutton is used to vary the delay length on-the-fly.                                    *
*   IRQ2 selects between either Slapback or Stereo Doubling effects.                                *
*                                                                                                   *
*                                                                                                   *
*                                                                          John Tomarakos           *
*                                                                          ADI DSP Applications     *
*                                                                          Revision 2.0             *
*                                                                          4/28/98                  *
*                                                                                                   *
*****************************************************************************************************/

/* ADSP-21060 System Register bit definitions */
#include 	"def21065l.h"
#include 	"new65Ldefs.h"

.GLOBAL		Init_Delay_Buffer;
.GLOBAL		Slapback_Echo;
.GLOBAL		Stereo_Double_Tracking;
.GLOBAL		change_delay_settings;
.GLOBAL		change_audio_effect;
.GLOBAL		effects_counter;
.EXTERN		Left_Channel_In;
.EXTERN 	Right_Channel_In;
.EXTERN 	Left_Channel;
.EXTERN 	Right_Channel;
.EXTERN		RX_left_flag;

/* ------------- DATA MEMORY FILTER BUFFERS ---------------------------------*/
.segment /dm	dm_delay;

#define DelayLine  	1632			/* Depth, or TD = D/fs = 1680/48000 = 35 msec   */
#define DelayLine2  3000 			/* Depth, or TD = D/fs = 3000/48000 = 62.5 msec */
#define DelayLine3	6000			/* Depth, or TD = D/fs = 3000/48000 = 125 msec  */
#define DelayLine4	12000			/* Depth, or TD = D/fs = 3000/48000 = .25 sec   */
#define DelayLine5	24000			/* Depth, or TD = D/fs = 3000/48000 = .50 sec   */
#define DelayLine6	48000			/* Depth, or TD = D/fs = 3000/48000 = 1.0 sec   */
			
.var	w[DelayLine6 + 1];			/* delay-line buffer, max delay = D */

.endseg;

/* ----------- INTERRUPT/FLAG DELAY FX DEMO CONTROL PARAMETERS ---------*/
.segment /dm	IRQ_ctl;

/* Reverb Control Parameters, these control 'knobs' are used to change the response on-the-fly */
.VAR	delay_time = 1000;				/* controls length (L6 register)of predelay buffer */
										/* length value always << max buffer length! */

.VAR	IRQ1_counter = 0x00000004;		/* selects preset 1 first on IRQ1 assertion */
.VAR	effects_counter = 0x00000001;	/* default is slapback effect */

.endseg;

/* ---------------------------------- PROGRAM MEMORY CODE ------------------------------------- */

.segment /pm pm_code;

Init_Delay_Buffer:
	B6 = w;  	L6 = @w;     			/* delay-line buffer pointer and length for ADT Effect */
	M6 = 1;

	LCNTR = L6; 						/* clear delay line buffer 2 to zero */
    DO clrDline UNTIL LCE;
clrDline:         dm(I6, M6) = 0;

	rts;


/* ///////////////////////////////////////////////////////////////////////////////////////// */
/*      Slapback Echo - Mono Doubling Audio Effect (ADT) using a digital delay-line 	     */
/*      Digital Delay Effect to create a mono echo effect                                    */
/*                                                                                           */
/*      This routines scales & mixes both input channels into 1 audio stream                 */
/* ///////////////////////////////////////////////////////////////////////////////////////// */
	
Slapback_Echo:  						/* process both channel inputs */	   	
   	/* get input samples from data holders */
   	r0 = dm(Left_Channel_In);          	/* left input sample */
   	r1 = dm(Right_Channel_In);         	/* right input sample */

	r1 = ashift r1 by -1;				/* scale signal by 1/2 for equal mix */
	r2 = ashift r2 by -1;				/* scale signal by 1/2 for equal mix */
	r1 = r2 + r1;						/* 1/2xL(n) + 1/2 xR(n) */

	L6 = dm(delay_time);
   	/* tap output of circular delay line */
   	r3 = dm(i6, 0);    					/* point to d-th tap and put in data register */
										/* fetch address with no update */

	/* add delayed signal together with original signal */
    r1 = ashift r1 by -1;				/* scale input signal by 1/2 for equal mix */
	r3 = ashift r3 by -1;				/* scale delayed signal by 1/2 for equal mix */
	r4 = r3 + r1;						/* 1/2xL(n) + 1/2 xR(n) */

   	/* write output samples to SPORT0 I2S channel A transmitter */
	r4 = ashift r4 by 1;				/* turn up the volume a little */
   	dm(Left_Channel)  = r4;    			/* left output sample */
   	dm(Right_Channel) = r4;	  			/* right output sample */

   	/* put input sample into tap-0 of delay line, post-modify address after storage of input */
   	dm(i6, -1) = r1;  					/* put value from register r1 into delay line */
										/* and decrement address by -1 */
    r4 = 0;
	dm(RX_left_flag) = r4;				/* clear RX_left_flag since we have processed incoming data */

	rts;								/* Return from Subroutine */
	          		
	
/* ///////////////////////////////////////////////////////////////////////////////////////// */
/*      Stereo Automatic Double Tracking - ADT Audio Effect using a digital delay-line       */
/*      Digital Delay Effect to create a stereo field effect                                 */
/*      Also called 'Stereo Doubling'                                                        */
/* ///////////////////////////////////////////////////////////////////////////////////////// */

Stereo_Double_Tracking:  				/* process right channel input only */
	   	
   	/* get input samples from data holders */
   	r0 = dm(Left_Channel_In);           /* left input sample */
   	r1 = dm(Right_Channel_In);         	/* right input sample */

	L6 = dm(delay_time);

   	/* tap output of circular delay line */
   	r3 = dm(i6, 0);    					/* point to d-th tap and put in data register */
										/* fetch address with no update */

   	/* write output samples to SPORT0 I2S channel A transmitter */
   	dm(Left_Channel)  = r0;       	/* left output sample */
   	dm(Right_Channel) = r3;	     	/* right output sample */

   	/* put input sample into tap-0 of delay line, post-modify address after storage of input */
   	dm(i6, -1) = r1;  					/* put value from register r1 into delay line */
										/* and decrement address by -1 */
	r4 = 0;
	dm(RX_left_flag) = r4;				/* clear RX_left_flag since we have processed incoming data */

   	rts;								/* Return from Subroutine */


/* ------------------------------------------------------------------------------------ */
/*                                                                                      */
/*                      IRQ2 Pushbutton Interrupt Service Routine                       */
/*                                                                                      */
/* 	This routine determines which audio effect routine to execute.                      */
/* 	Pressing the IRQ2 pushbutton will toggle between each effect. The default           */
/*	effect slapback.                                                                    */
/*                                                                                      */
/* ------------------------------------------------------------------------------------ */

change_audio_effect:
    bit set mode1 SRRFH;				/* enable background register file */
	NOP; 								/* 1 CYCLE LATENCY FOR WRITING TO MODE1 REGISER!!   */

	r11 = 2;
	r10 = DM(effects_counter);			/* get last count from memory */
	r10 = r10 + 1;						/* increment preset */
	comp (r10, r11);					/* compare current count to max count */
	if ge r10 = r10 - r10;				/* if count equals max, reset to zero and start over */
	DM(effects_counter) = r10;			/* save updated count */
exit:	
	rti(db);
	bit clr mode1 SRRFH;				/* switch back to primary register set */
	nop;


/* ------------------------------------------------------------------------------------ */
/*                                                                                      */
/*                       IRQ1 Pushbutton Interrupt Service Routine                      */
/*                                                                                      */
/* 		This routine allows the user to modify reverb presets on-the-fly to allow for  	*/
/*      emulation of either large 'valley' echo, medium hallway echo, or small echo.	*/
/*                                                                                      */
/*      Default before 1st IRQ push:	Delay = 20.83 msec                              */
/*      1st Pushbutton Press:	Delay Setting #1 -   35 msec                            */
/*      2nd Pushbutton Press:	Delay Setting #2 - 62.5 msec                            */
/*      3rd Pushbutton Press:	Delay Setting #3 -  125 msec                            */
/*      4th Pushbutton Press:	Delay Setting #4 - 0.25 sec                             */
/*      5th Pushbutton Press:	Delay Setting #4 - 0.50 sec                             */
/*      6th Pushbutton Press:	Delay Setting #4 -  1.0 sec                             */
/*      7th Pushbutton Press:	Reverts back to 1st Pushbutton Press                    */	
/* ------------------------------------------------------------------------------------ */
						  
change_delay_settings:
    bit set mode1 SRRFH;				/* enable background register file */
	NOP; 								/* 1 CYCLE LATENCY FOR WRITING TO MODE1 REGISER!!   */

	/* if flag2_in jump change_parameter; */

	r13 = 6;							/* number of presets */	
	r15 = DM(IRQ1_counter);				/* get preset count */
	r15 = r15 + 1;						/* increment preset */
	comp (r15, r13);
	if ge r15 = r15 - r15;				/* reset to zero */
	DM(IRQ1_counter) = r15;				/* save preset count */

	r10 = pass r15;						/* get preset mode */
	if eq jump delay_settings_2;		/* check for count == 0 */
	r10 = r10 - 1;
	if eq jump delay_settings_3; 		/* check for count == 1 */
	r10 = r10 - 1;
	if eq jump delay_settings_4;		/* check for count == 3 */
	r10 = r10 - 1;
	if eq jump delay_settings_5;		/* check for count == 4 */
	r10 = r10 - 1;
	if eq jump delay_settings_6;		/* check for count == 4 */

delay_settings_1:						/* count therefore, is == 5 if you are here */
	/* Depth, or TD = D/fs = 1680/48000 = 35 msec   */
	r14 = 1632;	  
	DM(delay_time) = r14;
	bit set ustat1 0x3E;				/* turn on Flag4 LED */
	bit clr ustat1 0x01;
	dm(IOSTAT)=ustat1;
    jump done;

delay_settings_2:
	/* Depth, or TD = D/fs = 3000/48000 = 62.5 msec */
	r14 = 3000;	  
	DM(delay_time) = r14;
	bit set ustat1 0x3D;				/* turn on Flag5 LED */
	bit clr ustat1 0x02;
	dm(IOSTAT)=ustat1;		
    jump done;

delay_settings_3:
	/* Depth, or TD = D/fs = 6000/48000 = 125 msec  */			
	r14 = 6000;	  
	DM(delay_time) = r14;	
	bit set ustat1 0x3B;				/* turn on Flag6 LED */
	bit clr ustat1 0x04;
	dm(IOSTAT)=ustat1;
	jump done;

delay_settings_4:
	/* Depth, or TD = D/fs = 12000/48000 = .25 sec  */			
	r14 = 12000;	  
	DM(delay_time) = r14;
	bit set ustat1 0x37;				/* turn on Flag7 LED */
	bit clr ustat1 0x08;
	dm(IOSTAT)=ustat1;
	jump done;

delay_settings_5:
	/* Depth, or TD = D/fs = 24000/48000 = .50 sec  */			
	r14 = 24000;	  
	DM(delay_time) = r14;	
	bit set ustat1 0x2F;				/* turn on Flag8 LED */
	bit clr ustat1 0x10;
	dm(IOSTAT)=ustat1;
	jump done;

delay_settings_6:
	/* Depth, or TD = D/fs = 48000/48000 = 1.0 sec  */			
	r14 = 48000;	  
	DM(delay_time) = r14;	
	bit set ustat1 0x1F;				/* turn on Flag9 LED */
	bit clr ustat1 0x20;
	dm(IOSTAT)=ustat1;

done:	
	rti(db);
	bit clr mode1 SRRFH;				/* switch back to primary register set */
	nop;	
	          
.endseg;

